import React from 'react';
import PropTypes from 'prop-types';
import ChartWrapper from '../ChartWrapper/ChartWrapper';

// Import styles
import getStyles from './PieChart.styles';

/**
 * The pie chart is mainly used for showing proportion of different categories.
 * Each arc length represents the proportion of data quantity.
 * @see https://echarts.apache.org/examples/en/editor.html?c=pie-simple
 */
const PieChart = (props) => {
    const {
        legendOptions, // TODO: remove legendOptions as they are for testing
        animation,
        contextStyles,
        horizontal,
        legend,
        onPress,
        seriesData,
        testID,
        title,
        tooltip,
    } = props ;

    const styles = getStyles(contextStyles);

    // Map series data prop to chart data structure
    const mappedChartData = seriesData.map((item) => {
        // Begin pie detail
        const pieDetail = {
            itemStyle: {},
            name: item.label,
            value: item.value,
        };

        // Add Pie section color details
        if (item.borderColor) {
            pieDetail.itemStyle.borderColor = item.borderColor;
        }
        if (item.borderWidth || item.borderWidth === 0) {
            pieDetail.itemStyle.borderWidth = item.borderWidth;
        }
        if (item.color) {
            pieDetail.itemStyle.color = item.color;
        }
        if (item.opacity || item.opacity === 0) {
            pieDetail.itemStyle.opacity = item.opacity;
        }
        if (item.emphasisColor) {
            pieDetail.itemStyle.emphasis = { color: item.emphasisColor };
        }

        return pieDetail;
    });

    // Build chart options
    // Default to vertical layout with pie centered on top of possible legend
    let pieCenter = [
        '50%',
        '25%',
    ];

    // For horizontal layouts
    if (horizontal) {
        // Display pie to the left of the legend
        if (legend) {
            pieCenter = [
                '25%',
                '50%',
            ];
        }

        // No legend, center in viewport
        else {
            pieCenter = [
                '50%',
                '50%',
            ];
        }
    }

    const chartOptions = {
        animation,
        backgroundColor: 'transparent',
        title: {
            text: title,
            x: 'center',
        },
        series: [
            {
                type: 'pie',
                data: mappedChartData,
                radius: '75%',
                center: pieCenter,
                label: { show: false },
                labelLine: { show: false },
            },
        ],
    };

    // Set chart legend
    // Default to horizontal layout
    // TODO: figure out nice horizontal layout with many legend items
    const legendLocation = {
        right: '10%',
        bottom: '10%',
        top: '10%',
        left: '50%',
        orient: 'vertical',
        type: 'scroll',

        // TODO: remove legendOptions as they are for testing
        ...legendOptions,
    };

    // TODO: figure out nice vertical layout with many legend items
    if (!horizontal) {
        legendLocation.top = '55%';
        legendLocation.left = '10%';
        legendLocation.right = '10%';
        legendLocation.type = 'plain';
        legendLocation.orient = 'vertical';
    }
    if (legend) {
        const legendData = seriesData.map((item) => item.label);
        chartOptions.legend = {
            data: legendData,
            ...legendLocation,
        };
    }

    // Enable tooltip
    if (tooltip) {
        chartOptions.tooltip = {
            confine: true,
        };
    }

    // Generate props
    const chartProps = {
        contextStyles: {
            container: styles.container,
        },
        option: chartOptions,
    };
    if (onPress) { chartProps.onPress = onPress; }
    if (testID) { chartProps.testID = testID; }

    return (
        <ChartWrapper { ...chartProps } />
    );
};

PieChart.propTypes = {
    // TODO: remove legendOptions as they are for testing
    legendOptions: PropTypes.object,

    /** Enables/disables chart animations */
    animation: PropTypes.bool,

    /** Styles for the component */
    contextStyles: PropTypes.shape({
        /** Styles for the container */
        container: PropTypes.object,
    }),

    /** Display horizontal with legend on right */
    horizontal: PropTypes.bool,

    /** Displays the information for the charts */
    legend: PropTypes.bool,

    /**
     * Triggered on press of a pie section
     * @param {Object} itemData - Data for the pie section pressed
     */
    onPress: PropTypes.func,

    /** The data for the pie chart */
    seriesData: PropTypes.arrayOf(PropTypes.shape({
        /** Border color of the pie section */
        borderColor: PropTypes.string,

        /** Border width of the pie section */
        borderWidth: PropTypes.number,

        /** Color of the pie section */
        color: PropTypes.string,

        /** Highlight color of the pie on press/hover */
        emphasisColor: PropTypes.string,

        /** Name of the sliced data */
        label: PropTypes.string.isRequired,

        /** Opacity of the pie section */
        opacity: PropTypes.number,

        /** The value of the pie section */
        value: PropTypes.number.isRequired,
    })).isRequired,

    /** Id used for testing */
    testID: PropTypes.string,

    /** Title text for the chart */
    title: PropTypes.string,

    /** Show related data in a tooltip after pressing the pie section */
    tooltip: PropTypes.bool,
};

PieChart.defaultProps = {
    animation: true,
    legend: true,
    contextStyles: {},
    tooltip: true,

    // TODO: remove legendOptions as they are for testing
    legendOptions: {
        right: '10%',
        bottom: '10%',
        top: '10%',
        left: '50%',
        orient: 'vertical',
        type: 'scroll',
    },
};

export default PieChart;
